﻿using Microsoft.AspNet.Mvc;
using HeuristicLab.Clients.Hive.WebJobManager.Services;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using HeuristicLab.Clients.Hive.WebJobManager.ViewModels;
using System.ServiceModel.Security;
using Microsoft.AspNet.Http;
using System.IO;
using Microsoft.Net.Http.Headers;
using Microsoft.AspNet.Hosting;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Optimization;

namespace HeuristicLab.Clients.Hive.WebJobManager.Controllers
{
    public class JobController : Controller
    {
        private HiveServiceClient client;
        private JobViewModel vm;
        private IHostingEnvironment _environment;

        public JobController(IHostingEnvironment env)
        {
            HiveServiceLocatorWebManagerService hiveServiceLocator = (HiveServiceLocatorWebManagerService)HiveServiceLocatorWebManagerService.Instance;
            client = hiveServiceLocator.getHiveServiceClient();
            vm = new JobViewModel();


            _environment = env;
        }
        public IActionResult Index()
        {
            try
            {
                vm.userJobs = client.GetJobs();
            }
            catch (Exception e)
            {
                if (e is MessageSecurityException || e is InvalidOperationException)
                {
                    HiveServiceLocatorWebManagerService.SetLoginErrorMessage();
                    return RedirectToAction("Index", "Home");
                }
                throw;
            }
            ViewBag.Title = "Jobs";
            return View(vm);
        }
        public IActionResult Selected(Guid id)
        {
            if (((HiveServiceLocatorWebManagerService)(HiveServiceLocatorWebManagerService.Instance)).CheckLogin())
            {
                vm.userJobs = client.GetJobs();
                vm.selectedJob = client.GetJob(id);
                vm.jobTasks = client.GetLightweightJobTasks(id);
                ViewBag.Title = vm.selectedJob.Name + " - Jobs";
                return View("Index", vm);
            }
            else
            {
                HiveServiceLocatorWebManagerService.SetLoginErrorMessage();
                return RedirectToAction("Index", "Home");
            }
        }
        public IActionResult Delete(Guid id)
        {
            if (((HiveServiceLocatorWebManagerService)(HiveServiceLocatorWebManagerService.Instance)).CheckLogin())
            {

                vm.message = client.GetJob(id).Name + " deleted";
                client.DeleteJob(id);
                vm.userJobs = client.GetJobs();
                ViewBag.Title = "Jobs";
                return View("Index", vm);
            }
            else
            {
                HiveServiceLocatorWebManagerService.SetLoginErrorMessage();
                return RedirectToAction("Index", "Home");
            }
        }

        public IActionResult Upload()
        {
            if (((HiveServiceLocatorWebManagerService)(HiveServiceLocatorWebManagerService.Instance)).CheckLogin())
            {
                ViewBag.Title = "Upload Jobs";
                return View("Upload");
            }
            else
            {
                HiveServiceLocatorWebManagerService.SetLoginErrorMessage();
                return RedirectToAction("Index", "Home");
            }
        }
        [HttpPost]
        public async Task<IActionResult> Upload(ICollection<IFormFile> files)
        {

            UploadedJobViewModel upper = new UploadedJobViewModel();
            var uploads = Path.Combine(_environment.WebRootPath, "uploads");
            foreach (var file in files)
            {
                if (file.Length > 0)
                {
                    var fileName = ContentDispositionHeaderValue.Parse(file.ContentDisposition).FileName.Trim('"');
                    await file.SaveAsAsync(Path.Combine(uploads, fileName));
                    var ioptimizer = ContentManager.Load(Path.Combine(uploads, fileName));
                    OptimizerHiveTask task = new OptimizerHiveTask((IOptimizer)ioptimizer);
                    upper.Tasks.Add(task);
                }
            }
            ViewBag.Title = "Upload complete";
            return View();

        }
    }
}
